%
%   MFA Dynamics Toolbox
%
%   function [FullStock,Demand,Waste,StockChange] = CalculateFullStock_Normal_DetailedWaste(Stock,Tau,Sigma1)
%
%   Stefan Pauliuk, 2009
%
%   given a time series of a stock, this function calculates the time series
%   of input and waste generation, assuming normally distributed lifetime
%   with sigma and tau.
%   In addition we split the stock into cohort so that we can trace each
%   vintage class.
%   Time is discrete with an intervall size of one year.
%
%   For further information, check with the manual of the MFA Dynamics
%   library. 

function [FullStock,Demand,Waste,StockChange,DetailedCarsToWaste] = CalculateFullStock_Normal_DetailedWaste(Stock,Tau,Sigma1)

% 0)  Check for Sigma. If the Tau/Sigma-Ratio is bad, we will get useless
% results because we only consider time steps of one year.
if mean(Sigma1) < 0.1 % The limit of 0.1 has been chosen rather arbitrarily. The result of the function normpdf still sums up to 1 for such a sigma and one year interval.
    disp('Standard deviation is too low! Function CalculateStock_Normal cannot run.');
    FullStock = NaN;
    Demand = NaN;
    Waste = NaN;
    StockChange = NaN;
    return;
end

if sum((Sigma1 - 0.5 * Tau) > 0) > 0 % Here we would have considerable negative distribution shares which are excluded from the outset. We cut of the tail for negative lifetimes and normalise the remaining part to one again.
    disp('Warning: Standard deviation/lifetime is larger than 0.5, truncation probability measure is more than 2%.');
end


% 1) Calculate stock derivative
        nums = length(Stock); % This is the length of the time series.
        FullStock = zeros(nums,nums); % Define full stock table.

% Calculate the derivative of the stock:

        Stockdiff = zeros(nums,1);
        Stockdiff(2:end) = Stock(1:end-1);

        % StockChange = Stock(t) - Stock(t-1)
        StockChange = Stock - Stockdiff; % This is the discrete derivative.
        
        

% 2) Now, we build up the stock by its in- and outflows:
        Demand = zeros(nums,1);
        Waste = zeros(nums,1);

%get normally distributed values that we use as lifetime model:
        safetimelength = max(floor(max(Tau)+5*max(Sigma1)), nums);
        X = (1:1:safetimelength)'; % Make a long X to be shure to have a very little tail cut.
        PDFs = zeros(length(Stock),safetimelength);
        for m = 1:1:length(Stock)
            PDFs(m,:) = normpdf(X,Tau(m),Sigma1(m)); % This is the only place where Tau and Sigma1 enter explicitly
        end
        

% ! Negative lifetimes do not extist and limit the application of this
% model. If Sigma1 is more than a certain percentage of tau, the function
% will give a warning. However it may happen before that sum(PDF) is not one. For
% these cases we correct the values:
        SumPDFs = sum(PDFs,2);
        PDFs = PDFs ./ repmat(SumPDFs,1,safetimelength);

% For each year, we calculate the hypothetical output first.
% This is done by multiplying all historic inputs with the PDF

DetailedCarsToWaste = zeros(nums,nums);

% 2.1) First year:

        Waste(1) = 0;
        Demand(1) = StockChange(1) + Waste(1); 
        FullStock(1,1) = Demand(1);


% 2.2) All other years

        for n = 2:nums
            %Age = (n-1:-1:1)'; % Age of cohorts, seen from time step n
                thiswaste = 0;
                for m = 1:n-1
                    FullStock(n,m) =  FullStock(n-1,m) - (PDFs(m,n-m) * Demand(m));
                    thiswaste = thiswaste + (PDFs(m,n-m) * Demand(m));
                    DetailedCarsToWaste(n,m) = (PDFs(m,n-m) * Demand(m));
                end
                Waste(n) = thiswaste;
            %FullStock(n,1:n-1) =  FullStock(n-1,1:n-1) - (PDFs() .* Demand(1:n-1))';
            %Waste(n) = sum(PDFs(n,Age) .* Demand(1:n-1)); 
            
            % the initial stock as well as all other cohorts contribute
            if StockChange(n) < - Waste(n) % Check whether Stock is decreasing too fast.
                disp('<b>Stock decreasing too fast! Function CalculateFullStock_Normal cannot continue.</b><br>');
                Demand = NaN;
                Waste = NaN;
                StockChange = NaN;
                return;
            else
             Demand(n) = StockChange(n) + Waste(n); % calculate demand from StockChange and Waste
             FullStock(n,n) = Demand(n);
           end
        end


disp('Function CalculateFullStock_Normal_DetailedWaste finished.<br>');

end